extern "C"{
#include "mex.h"
#include <f2c.h>
#include <clapack.h>
}

#include <iostream>
#include <cstring>
/*
#include <math.h>
*/

/* GAUSS interface */
extern "C"{
  void mjdgges(double *a, double *b, double *z, double *n, double *sdim, double *eval_r, double *eval_i, double *info);
}

logical my_criteria(double *alphar, double *alphai, double *beta)
{
  return( (*alphar * *alphar + *alphai * *alphai) < *beta * *beta);
}

void mjdgges(double *a, double *b, double *z, double *n, double *sdim, double *eval_r, double *eval_i, double *info)
{
  integer i_n, i_info, i_sdim;
  i_n = (integer)*n;
  integer one = 1;
  double *alphar = new double[i_n];
  double *alphai = new double[i_n];
  double *beta = new double[i_n];
  integer lwork = 16*i_n+16;
  double *work = new double[lwork];
  logical *bwork = new logical[i_n];
  double junk;

  dgges_( "N", "V", "S", (L_fp)my_criteria, &i_n, a, &i_n, b, &i_n, &i_sdim, alphar, alphai, beta, &junk, &one, z, &i_n, work, &lwork, bwork, &i_info );
  
  *sdim = i_sdim;
  *info = i_info;

  double *par = alphar;
  double *pai = alphai;
  double *pb = beta;
  double *pei = eval_i;
  for(double *per = eval_r; per <= &eval_r[i_n-1]; ++per)
    {
      *per = *par / *pb;
      *pei = *pai / *pb;
      ++par;
      ++pai;
      ++pb;
      ++pei;
    }
}

/* MATLAB interface */
void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray *prhs[] )
     
{ 
  unsigned int m1,n1,m2,n2; 
    
    /* Check for proper number of arguments */
    
    if (nrhs != 2) { 
	mexErrMsgTxt("Three input arguments required."); 
    } else if (nlhs > 6) {
	mexErrMsgTxt("Too many output arguments."); 
    } 
    
    /* Check that A and B are real matrices of the same dimension.*/ 
    
    m1 = mxGetM(prhs[0]); 
    n1 = mxGetN(prhs[0]);
    m2 = mxGetM(prhs[1]); 
    n2 = mxGetN(prhs[1]);
    if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) || 
	!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) || 
	(m1 != n1) || (m2!= n1) || (m2 != n2)) { 
      mexErrMsgTxt("MYDGGES requires two square real matrices of the same dimension."); 
    } 
    
    /* Create a matrix for the return argument */ 
    plhs[0] = mxCreateDoubleMatrix(n1, n1, mxREAL); 
    plhs[1] = mxCreateDoubleMatrix(n1, n1, mxREAL); 
    plhs[2] = mxCreateDoubleMatrix(n1, n1, mxREAL); 
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL); 
    plhs[4] = mxCreateDoubleMatrix(n1, 1, mxCOMPLEX); 
    plhs[5] = mxCreateDoubleMatrix(1, 1, mxREAL); 
    
    /* Assign pointers to the various parameters */ 
    double *s = mxGetPr(plhs[0]);
    double *t = mxGetPr(plhs[1]);
    double *z = mxGetPr(plhs[2]);
    double *sdim = mxGetPr(plhs[3]);
    double *eval_r = mxGetPr(plhs[4]);
    double *eval_i = mxGetPi(plhs[4]);
    double *info = mxGetPr(plhs[5]);
    	    
    	    
    double *a = mxGetPr(prhs[0]);
    double *b = mxGetPr(prhs[1]);

    /* keep a and b intact */
    memcpy(s,a,sizeof(double)*n1*n1);
    memcpy(t,b,sizeof(double)*n1*n1);

    double n = n1;

    /* Do the actual computations in a subroutine */
    mjdgges(s, t, z, &n, sdim, eval_r, eval_i, info);


    return;
    
}

/*
int main(int argi, char **argc)
{
  double a[] = { 0, 3, 0, 7, 0,
		 1, 0, 0, 0, 0,
		 2, 4, 0, 0, 0,
		 0, 0, 6, 0, 0,
		 0, 5, 0, 0, 8};

  double b[] = { 1,  0,  0,  0,  7,
		 0, -2,  0,  0,  0,
		 0,  0,  3,  0, -8,
		 0, -4,  5,  0,  0,
		 1,  0,  0, -6,  0};
  
  double n = 5;
  int i_n = (int)n;
  double info, sdim;

  double *eval_r, *eval_i, *z;
  eval_r = new double[i_n];
  eval_i = new double[i_n];
  z = new double[i_n*i_n];

  mjdgges(a, b, z, &n, &sdim, eval_r, eval_i, &info);
 
  cout << info << "\n";
  cout << sdim << "\n";
 
  cout << "\n";
  for (int i =0; i < n; ++i)
    {
      cout << eval_r[i] << "   " << eval_i[i] << "\n";
    }

  double *pz = z;
  for (int i =0; i < n; ++i)
    {
      for (int j = 0; j < n; ++j)
	{
	  cout << *pz << "   ";
	  ++pz;
	}
      cout << "\n";
    }

  cout << "\n";

  return 0;
}
*/











